@@ -64,9 +64,15 @@ expect_error(vars_range("d_1", "d_5"))
 
 context("%to%")
 expect_identical(a_1 %to% a_5, data.frame(a_1 = a_1, a_2 = a_2, a_4 = a_4, a_5 = a_5))
+expect_identical(a_1 %to_list% a_5, list(a_1 = a_1, a_2 = a_2, a_4 = a_4, a_5 = a_5))
 expect_identical(
     with(dfs, a_1 %to% a_5), 
     data.frame(a_1 = a_1, a_2 = a_2, a_4 = a_4, a_5 = a_5))
+
+expect_identical(
+    with(dfs, a_1 %to_list% a_5), 
+    list(a_1 = a_1, a_2 = a_2, a_4 = a_4, a_5 = a_5))
+
 expect_identical(
     with(dfs, b_1 %to% b_5), 
     with(dfs, data.frame(b_1 = b_1, b_2 = b_2, b_4 = b_4, b_5 = b_5))
@@ -96,8 +102,8 @@ expect_error(a_1a %to% a_5)
 expect_error(rep("a_1",2) %to% rep("a_5",2))
 expect_error(d_1 %to% d_5)
 
-context("dplyr")
-if(suppressWarnings(require(dplyr, quietly = TRUE))){
+# context("dplyr")
+# if(suppressWarnings(require(dplyr, quietly = TRUE))){
     # expect_identical(
     #     dfs %>% mutate(
     #               b_total = sum_row(vars_range("b_1", "b_5")),
@@ -113,9 +119,9 @@ if(suppressWarnings(require(dplyr, quietly = TRUE))){
     #     
     # )
     
-} else {
-    cat("dplyr not found\n")
-}
+# } else {
+    # cat("dplyr not found\n")
+# }
 
 context("magrittr")
 if(suppressWarnings(require(magrittr, quietly = TRUE))){
@@ -254,9 +260,15 @@ expect_error(vars_pattern("d_[0-9]"))
 
 context("vars")
 expect_identical(vars("a_`c(1:2,4:5)`"), data.frame(a_1 = a_1, a_2 = a_2, a_4 = a_4, a_5 = a_5))
+expect_identical(vars_list("a_`c(1:2,4:5)`"), list(a_1 = a_1, a_2 = a_2, a_4 = a_4, a_5 = a_5))
 expect_identical(
     with(dfs, vars("a_`c(1:2,4:5)`")), 
     data.frame(a_1 = a_1, a_2 = a_2, a_4 = a_4, a_5 = a_5))
+
+expect_identical(
+    with(dfs, vars_list("a_`c(1:2,4:5)`")), 
+    list(a_1 = a_1, a_2 = a_2, a_4 = a_4, a_5 = a_5))
+
 expect_identical(
     with(dfs, vars("b_`c(1:2,4:5)`")), 
     with(dfs, data.frame(b_1 = b_1, b_2 = b_2, b_4 = b_4, b_5 = b_5))
